# -*- coding: utf-8 -*-
# cython: boundscheck=False, wraparound=False, profile=True, linetrace=True, binding=True


cimport cython
import numpy
cimport numpy
numpy.import_array()

from numpy.linalg import norm


def masked_euclidean(numpy.ndarray[numpy.double_t, ndim=2] data):
    cdef Py_ssize_t m = data.shape[0]
    cdef Py_ssize_t i = 0
    cdef Py_ssize_t j = 0
    cdef Py_ssize_t k = 0
    cdef numpy.ndarray[numpy.double_t] dm = numpy.zeros(m * (m - 1) // 2, dtype=numpy.double)
    cdef numpy.ndarray[numpy.uint8_t, ndim=2, cast=True] mask = numpy.isfinite(data) # used as boolean
    cdef numpy.ndarray[numpy.uint8_t, cast=True] vec_mask = numpy.zeros(data.shape[1], dtype=bool) # used as boolean
    for i in range(m - 1):
        for j in range(i + 1, m):
            vec_mask = mask[i] * mask[j]
            dm[k] = norm(data[i][vec_mask] - data[j][vec_mask])
            k += 1
    return dm

